@@ -23,9 +23,9 @@ use crate::fce_test::config_utils;
 use fluence_app_service::TomlAppServiceConfig;
 use proc_macro2::TokenStream;
 use quote::quote;
-use quote::ToTokens;
 
 use std::path::Path;
+use std::path::PathBuf;
 
 /// Generates glue code for tests.
 /// F.e. for this test for the greeting service
@@ -114,14 +114,18 @@ use std::path::Path;
 pub(super) fn generate_test_glue_code(
     func_item: syn::ItemFn,
     attrs: FCETestAttributes,
+    file_path: PathBuf,
 ) -> TResult<TokenStream> {
-    let fce_config = TomlAppServiceConfig::load(&attrs.config_path)?;
+    let config_path = file_path.join(&attrs.config_path);
+
+    let fce_config = TomlAppServiceConfig::load(&config_path)?;
     let modules_dir = match config_utils::resolve_modules_dir(&fce_config, attrs.modules_dir) {
         Some(modules_dir) => modules_dir,
         None => return Err(TestGeneratorError::ModulesDirUnspecified),
     };
 
-    let app_service_ctor = generate_app_service_ctor(&attrs.config_path, &modules_dir);
+    let app_service_ctor = generate_app_service_ctor(&attrs.config_path, &modules_dir)?;
+    let modules_dir = file_path.join(modules_dir);
     let module_interfaces = fce_test::config_utils::collect_modules(&fce_config, modules_dir)?;
 
     let module_definitions =
@@ -153,11 +157,12 @@ pub(super) fn generate_test_glue_code(
     Ok(glue_code)
 }
 
-fn generate_app_service_ctor(config_path: &str, modules_dir: &Path) -> TokenStream {
-    let config_path = config_path.to_token_stream();
-    let modules_dir = modules_dir.to_string_lossy().to_string();
+fn generate_app_service_ctor(config_path: &str, modules_dir: &Path) -> TResult<TokenStream> {
+    let modules_dir = modules_dir
+        .to_str()
+        .ok_or_else(|| TestGeneratorError::InvalidUTF8Path(modules_dir.to_path_buf()))?;
 
-    quote! {
+    let service_ctor = quote! {
         let tmp_dir = std::env::temp_dir();
         let service_id = fluence_test::internal::Uuid::new_v4().to_string();
 
@@ -165,16 +170,51 @@ fn generate_app_service_ctor(config_path: &str, modules_dir: &Path) -> TokenStre
         let tmp_dir = tmp_dir.to_string_lossy().to_string();
         std::fs::create_dir(&tmp_dir).expect("can't create a directory for service in tmp");
 
-        let mut __fce_generated_fce_config = fluence_test::internal::TomlAppServiceConfig::load(#config_path.to_string())
-            .unwrap_or_else(|e| panic!("app service located at `{}` config can't be loaded: {}", #config_path, e));
+        let mut module_path = std::path::PathBuf::from(env!("CARGO_MANIFEST_DIR"));
+        let mut file_path = std::path::Path::new(file!()).components();
+
+        let mut truncated_file_path = Vec::new();
+        loop {
+            if module_path.ends_with(file_path.as_path()) {
+                break;
+            }
+
+            let (file_path_, remainder) = match file_path.next_back().and_then(|p| match p {
+                std::path::Component::Normal(_) | std::path::Component::CurDir | std::path::Component::ParentDir => {
+                    Some((file_path, p))
+                }
+                _ => None,
+            }) {
+                Some(t) => t,
+                None => break,
+            };
+            file_path = file_path_;
+
+            truncated_file_path.push(remainder);
+        }
+
+        for path in truncated_file_path.iter().rev() {
+            module_path.push(path);
+        }
+
+        let _ = module_path.pop();
+
+        let config_path = module_path.join(#config_path);
+        let modules_dir = module_path.join(#modules_dir);
+        let modules_dir = modules_dir.to_str().expect("modules_dir contains invalid UTF8 string");
+
+        let mut __fce_generated_fce_config = fluence_test::internal::TomlAppServiceConfig::load(&config_path)
+            .unwrap_or_else(|e| panic!("app service config located at `{:?}` can't be loaded: {}", config_path, e));
         __fce_generated_fce_config.service_base_dir = Some(tmp_dir);
-        __fce_generated_fce_config.toml_faas_config.modules_dir = Some(#modules_dir.to_string());
+        __fce_generated_fce_config.toml_faas_config.modules_dir = Some(modules_dir.to_string());
 
         let fce = fluence_test::internal::AppService::new_with_empty_facade(__fce_generated_fce_config, service_id, std::collections::HashMap::new())
             .unwrap_or_else(|e| panic!("app service can't be created: {}", e));
 
         let fce = std::rc::Rc::new(std::cell::RefCell::new(fce));
-    }
+    };
+
+    Ok(service_ctor)
 }
 
 fn generate_module_ctors<'n>(
